/*
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "RendererRelay.hh"
#include "RendererRelayImpl.hh"
#include "SceneTreePtr.hh"
#include "SceneTree.hh"


using namespace peekabot;


RendererRelay::RendererRelay()
    : m_impl(new Impl(TheSceneTree::acquire()->get_root()))
{
}


RendererRelay::RendererRelay(const SceneObject *root)
    : m_impl(new Impl(root))
{
}


RendererRelay::~RendererRelay()
{
    delete m_impl;
    m_impl = 0;
}


void RendererRelay::render(
    unsigned int w, unsigned int h, ObjectID camera_id, 
    bool *visible_layers, renderer::FrameCollector *collector)
{
    m_impl->render(
        w, h, 
        camera_id,
        visible_layers, 
        collector);
}


uint32_t RendererRelay::pick(
    unsigned int w, unsigned int h, ObjectID camera_id,
    bool *visible_layers,
    int x, int y, float sensitivity)
{
    return m_impl->pick(
        w, h, 
        camera_id,
        visible_layers, 
        x, y, sensitivity);
}


void RendererRelay::initialize_renderer()
{
    m_impl->initialize_renderer();
}


bool RendererRelay::has_pending_renderer_updates() const throw()
{
    return m_impl->has_pending_renderer_updates();
}


void RendererRelay::process_renderer_updates()
{
    m_impl->process_renderer_updates();
}


renderer::api::Renderer &RendererRelay::get_renderer()
{
    return m_impl->get_renderer();
}


const renderer::api::Renderer &RendererRelay::get_renderer() const
{
    return m_impl->get_renderer();
}
