/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * Distributed under the Boost Software License, Version 1.0.
 * (See accompanying file LICENSE_1_0.txt or copy at
 * http://www.boost.org/LICENSE_1_0.txt)
 */

#include <stdexcept>
#include "SetSelected.hh"

#ifdef __PEEKABOT_SERVER
#  include "../ServerExecutionContext.hh"
#  include "../SceneObject.hh"
#endif


using namespace peekabot;


SetSelected::SetSelected() throw()
{
}


SetSelected::SetSelected(ObjectID target, bool select) throw()
    : m_target(target), 
      m_select(select)
{
}


Action *SetSelected::clone() const
{
    return new SetSelected(m_target, m_select);
}


void SetSelected::execute(ServerExecutionContext *context) const
{
#ifdef __PEEKABOT_SERVER
    SceneObject *ptr = context->get_object(m_target);

    if( !ptr )
        throw std::runtime_error("Target object not found");

    ptr->set_selected(m_select);
#endif
}


void SetSelected::save(SerializationInterface &ar) const
{
    ar << m_target << m_select;
}

void SetSelected::load(DeserializationInterface &ar)
{
    ar >> m_target >> m_select;
}
