/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdexcept>

#include "Scene.hh"
#include "Entity.hh"


using namespace peekabot;
using namespace peekabot::renderer;


void Scene::add_entity(EntityID id, Entity *entity)
{
    if( has_entity(id) )
        throw std::runtime_error(
            "The given EntityID is already in the scene");

    if( has_entity(entity) )
        throw std::runtime_error(
            "The given Entity is already in the scene");

    m_id_to_entity_map[id] = entity;
    m_entity_to_id_map[entity] = id;

    m_on_entity_added(entity);
}


void Scene::remove_entity(Entity *entity)
{
    if( !has_entity(entity) )
        throw std::runtime_error("There is no such entity in the scene");

    EntityToIDMap::iterator it = m_entity_to_id_map.find(entity);
    assert( it != m_entity_to_id_map.end() );
    EntityID id = it->second;
    m_entity_to_id_map.erase(it);

    IDToEntityMap::iterator jt = m_id_to_entity_map.find(id);
    assert( jt != m_id_to_entity_map.end() );
    m_id_to_entity_map.erase(jt);

    m_on_entity_removed(entity);

    delete entity;
}


Entity *Scene::get_entity(EntityID id)
{
    IDToEntityMap::iterator it = m_id_to_entity_map.find(id);
    if( it == m_id_to_entity_map.end() )
        throw std::runtime_error(
            "There's no entity with the given ID in the scene");
    else
        return it->second;
}


const Entity *Scene::get_entity(EntityID id) const
{
    IDToEntityMap::const_iterator it = m_id_to_entity_map.find(id);
    if( it == m_id_to_entity_map.end() )
        throw std::runtime_error(
            "There's no entity with the given ID in the scene");
    else
        return it->second;
}


bool Scene::has_entity(Entity *entity) const throw()
{
    EntityToIDMap::const_iterator it = m_entity_to_id_map.find(entity);
    return it != m_entity_to_id_map.end();
}


bool Scene::has_entity(EntityID id) const throw()
{
    IDToEntityMap::const_iterator it = m_id_to_entity_map.find(id);
    return it != m_id_to_entity_map.end();
}
