/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_RENDERER_SCENE_HH_INCLUDED
#define PEEKABOT_RENDERER_SCENE_HH_INCLUDED


#include <map>
#include <stdexcept>
#include <boost/thread_safe_signal.hpp>

#include "Types.hh"


namespace peekabot
{
    namespace renderer
    {
        class Entity;

        class Scene
        {
        public:
            void add_entity(EntityID id, Entity *entity);

            void remove_entity(Entity *entity);

            Entity *get_entity(EntityID id);

            const Entity *get_entity(EntityID id) const;

            template<class T> T *get_entity(EntityID id)
            {
                Entity *e = get_entity(id);
                T *t = dynamic_cast<T *>(e);
                if( !t )
                    throw std::runtime_error("Entity is of wrong type");
                else
                    return t;
            }

            template<class T> const T *get_entity(EntityID id) const
            {
                const Entity *e = get_entity(id);
                const T *t = dynamic_cast<const T *>(e);
                if( !t )
                    throw std::runtime_error("Entity is of wrong type");
                else
                    return t;
            }

            bool has_entity(Entity *entity) const throw();

            bool has_entity(EntityID id) const throw();


            /// \name Signals
            /// @{

            typedef boost::signal<void (const Entity *)> OnEntityAdded;
            mutable OnEntityAdded m_on_entity_added;

            typedef boost::signal<void (const Entity *)> OnEntityRemoved;
            mutable OnEntityRemoved m_on_entity_removed;

            /// @}

        private:
            typedef std::map<EntityID, Entity *> IDToEntityMap;
            IDToEntityMap m_id_to_entity_map;

            typedef std::map<Entity *, EntityID> EntityToIDMap;
            EntityToIDMap m_entity_to_id_map;
        };
    }
}


#endif // PEEKABOT_RENDERER_SCENE_HH_INCLUDED
