/*
 * Copyright Staffan Gimåker 2006-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_RENDERER_SCENE_ACTION_HH_INCLUDED
#define PEEKABOT_RENDERER_SCENE_ACTION_HH_INCLUDED


#include <boost/utility.hpp>


namespace peekabot
{
    namespace renderer
    {
        class Scene;
        class SceneManager;
        class RenderEngine;

        /**
         * \brief Command pattern interface for making changes to the scene.
         */
        class SceneAction
        {
        public:
            class Context : public boost::noncopyable
            {
                friend class RenderEngine;

            public:
                inline Scene &scene()
                {
                    return m_scene;
                }

                inline const Scene &scene() const
                {
                    return m_scene;
                }

                inline SceneManager &scene_manager()
                {
                    return m_scene_manager;
                }

                inline const SceneManager &scene_mananager() const
                {
                    return m_scene_manager;
                }

            private:
                Context(Scene &scene, SceneManager &scene_manager)
                    : m_scene(scene), m_scene_manager(scene_manager) {}

            private:
                Scene &m_scene;

                SceneManager &m_scene_manager;
            };


            virtual ~SceneAction() {}

            /**
             * \brief Execute out the encapsulated action.
             */
            virtual void execute(Context &context) = 0;
        };
    }
}


#endif // PEEKABOT_RENDERER_SCENE_ACTION_HH_INCLUDED
