/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CameraProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../entities/Camera.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


CameraProxy::CameraProxy(Renderer &renderer)
    : ObjectEntityProxy(renderer)
{
    class AddCamera : public SceneAction
    {
    public:
        AddCamera(EntityID id) : m_id(id) {}

        virtual void execute(SceneAction::Context &context)
        {
            Camera *cam = new Camera();

            try
            {
                context.scene().add_entity(m_id, cam);
            }
            catch(...)
            {
                delete cam;
                throw;
            }
        }

    private:
        const EntityID m_id;
    };

    dispatch(new AddCamera(entity_id()));
}


void CameraProxy::set_fov(float fov)
{
    class SetFov : public SceneAction
    {
    public:
        SetFov(EntityID id, float fov) : m_id(id), m_fov(fov) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Camera>(m_id)->set_fov(m_fov);
        }

    private:
        const EntityID m_id;
        const float m_fov;
    };

    dispatch(new SetFov(entity_id(), fov));
}


void CameraProxy::set_zoom_distance(float zoom_distance)
{
    class SetZoomDistance : public SceneAction
    {
    public:
        SetZoomDistance(EntityID id, float zoom_distance)
            : m_id(id), m_zoom_distance(zoom_distance) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Camera>(
                m_id)->set_zoom_distance(m_zoom_distance);
        }

    private:
        const EntityID m_id;
        const float m_zoom_distance;
    };

    dispatch(new SetZoomDistance(entity_id(), zoom_distance));
}


void CameraProxy::set_orthographic(bool orthographic)
{
    class SetOrthographic : public SceneAction
    {
    public:
        SetOrthographic(EntityID id, bool orthographic)
            : m_id(id), m_orthographic(orthographic) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Camera>(
                m_id)->set_orthographic(m_orthographic);
        }

    private:
        const EntityID m_id;
        const bool m_orthographic;
    };

    dispatch(new SetOrthographic(entity_id(), orthographic));
}


void CameraProxy::set_near_plane(float near)
{
    class SetNear : public SceneAction
    {
    public:
        SetNear(EntityID id, float near) : m_id(id), m_near(near) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Camera>(m_id)->set_near_plane(m_near);
        }

    private:
        const EntityID m_id;
        const float m_near;
    };

    dispatch(new SetNear(entity_id(), near));
}


void CameraProxy::set_far_plane(float far)
{
    class SetFar : public SceneAction
    {
    public:
        SetFar(EntityID id, float far) : m_id(id), m_far(far) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Camera>(m_id)->set_far_plane(m_far);
        }

    private:
        const EntityID m_id;
        const float m_far;
    };

    dispatch(new SetFar(entity_id(), far));
}
