/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "EntityProxy.hh"
#include "Renderer.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


EntityProxy::EntityProxy(Renderer &renderer)
    : m_renderer(renderer),
      m_entity_id(allocate_entity_id())
{
}


EntityProxy::~EntityProxy()
{
    class RemoveEntity : public SceneAction
    {
    public:
        RemoveEntity(EntityID id) : m_id(id) {}

        virtual void execute(SceneAction::Context &context)
        {
            if( context.scene().has_entity(m_id) )
            {
                Entity *e = context.scene().get_entity(m_id);
                context.scene().remove_entity(e);
            }
        }

    private:
        const EntityID m_id;
    };

    dispatch(new RemoveEntity(entity_id()));
}


EntityID EntityProxy::allocate_entity_id() throw()
{
    static EntityID s_id_alloc = 0;
    return ++s_id_alloc;
}


void EntityProxy::dispatch(SceneAction *action)
{
    m_renderer.dispatch(action);
}


EntityID EntityProxy::entity_id() const
{
    return m_entity_id;
}
