/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "LabelProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../entities/Label.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


LabelProxy::LabelProxy(Renderer &renderer)
    : ObjectEntityProxy(renderer)
{
    class AddLabel : public SceneAction
    {
    public:
        AddLabel(EntityID id) : m_id(id) {}

        virtual void execute(SceneAction::Context &context)
        {
            Entity *e = new Label();

            try
            {
                context.scene().add_entity(m_id, e);
            }
            catch(...)
            {
                delete e;
                throw;
            }
        }

    private:
        const EntityID m_id;
    };

    dispatch(new AddLabel(entity_id()));
}


void LabelProxy::set_text(const std::string &text)
{
    class SetText : public SceneAction
    {
    public:
        SetText(EntityID id, const std::string &text)
            : m_id(id), m_text(text) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Label>(m_id)->set_text(m_text);
        }

    private:
        const EntityID m_id;
        const std::string m_text;
    };

    dispatch(new SetText(entity_id(), text));
}


void LabelProxy::set_alignment(TextAlignment alignment)
{
    class SetAlignment : public SceneAction
    {
    public:
        SetAlignment(EntityID id, TextAlignment alignment)
            : m_id(id), m_alignment(alignment) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<Label>(
                m_id)->set_alignment(m_alignment);
        }

    private:
        const EntityID m_id;
        TextAlignment m_alignment;
    };

    dispatch(new SetAlignment(entity_id(), alignment));
}
