/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "LineBasedProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../LineBased.hh"
#include "../Entity.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


LineBasedProxy::LineBasedProxy(Renderer &renderer)
    : ObjectEntityProxy(renderer)
{
}


void LineBasedProxy::set_line_style(
    float line_width,
    uint16_t stipple_pattern,
    int stipple_factor)
{
    class SetLineStyle : public SceneAction
    {
    public:
        SetLineStyle(EntityID id,
                     float line_width,
                     uint16_t stipple_pattern,
                     int stipple_factor)
            : m_id(id),
              m_line_width(line_width),
              m_stipple_pattern(stipple_pattern),
              m_stipple_factor(stipple_factor) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<LineBased>(m_id)->set_line_style(
                m_line_width,
                m_stipple_pattern,
                m_stipple_factor);
        }

    private:
        const EntityID m_id;
        const float m_line_width;
        const uint16_t m_stipple_pattern;
        const int m_stipple_factor;
    };

    dispatch(
        new SetLineStyle(
            entity_id(), line_width, stipple_pattern, stipple_factor));
}
