/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ModelProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../entities/MeshBased.hh"
#include "../PbmfLoader.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


ModelProxy::ModelProxy(
    Renderer &renderer,
    const std::string &filename)
    : ObjectEntityProxy(renderer)
{
    class AddModel : public SceneAction
    {
    public:
        AddModel(EntityID id, const std::string &filename)
            : m_id(id), m_filename(filename) {}

        virtual void execute(SceneAction::Context &context)
        {
            PbmfLoader loader;
            MeshBased *model = loader.load(m_filename);

            try
            {
                context.scene().add_entity(m_id, model);
            }
            catch(...)
            {
                delete model;
                throw;
            }
        }

    private:
        const EntityID m_id;
        const std::string m_filename;
    };

    dispatch(new AddModel(entity_id(), filename));
}
