/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "ObjectEntityProxy.hh"
#include "Renderer.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../ObjectEntity.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


ObjectEntityProxy::ObjectEntityProxy(Renderer &renderer)
    : EntityProxy(renderer)
{
}


void ObjectEntityProxy::set_transformation(const Matrix4f &mtow)
{
    class SetTransformation : public SceneAction
    {
    public:
        SetTransformation(EntityID id, const Matrix4f &mtow)
            : m_id(id), m_mtow(mtow) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_transformation(m_mtow);
        }

    private:
        const EntityID m_id;
        const Matrix4f m_mtow;
    };

    dispatch(new SetTransformation(entity_id(), mtow));
}


void ObjectEntityProxy::set_color(const RGBColor &color)
{
    class SetColor : public SceneAction
    {
    public:
        SetColor(EntityID id, const RGBColor &color)
            : m_id(id), m_color(color) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_color(m_color);
        }

    private:
        const EntityID m_id;
        const RGBColor m_color;
    };

    dispatch(new SetColor(entity_id(), color));
}


void ObjectEntityProxy::set_opacity(float alpha)
{
    class SetOpacity : public SceneAction
    {
    public:
        SetOpacity(EntityID id, float alpha) : m_id(id), m_alpha(alpha) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_opacity(m_alpha);
        }

    private:
        const EntityID m_id;
        const float m_alpha;
    };

    dispatch(new SetOpacity(entity_id(), alpha));
}


void ObjectEntityProxy::set_layer(uint32_t layer)
{
    class SetLayer : public SceneAction
    {
    public:
        SetLayer(EntityID id, uint32_t layer) : m_id(id), m_layer(layer) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(m_id)->set_layer(m_layer);
        }

    private:
        const EntityID m_id;
        const uint32_t m_layer;
    };

    dispatch(new SetLayer(entity_id(), layer));
}


void ObjectEntityProxy::set_name(uint32_t name)
{
    class SetName : public SceneAction
    {
    public:
        SetName(EntityID id, uint32_t name) : m_id(id), m_name(name) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(m_id)->set_name(m_name);
        }

    private:
        const EntityID m_id;
        const uint32_t m_name;
    };

    dispatch(new SetName(entity_id(), name));
}


void ObjectEntityProxy::set_visibility(bool visible)
{
    class SetVisibility : public SceneAction
    {
    public:
        SetVisibility(EntityID id, bool visible) :
            m_id(id), m_visible(visible) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_visibility(m_visible);
        }

    private:
        const EntityID m_id;
        const bool m_visible;
    };

    dispatch(new SetVisibility(entity_id(), visible));
}


void ObjectEntityProxy::set_selected(bool is_selected)
{
    class SetSelected : public SceneAction
    {
    public:
        SetSelected(EntityID id, bool is_selected)
            : m_id(id), m_is_selected(is_selected) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_selected(m_is_selected);
        }

    private:
        const EntityID m_id;
        const bool m_is_selected;
    };

    dispatch(new SetSelected(entity_id(), is_selected));
}


void ObjectEntityProxy::set_has_selected_ancestor(bool has_selected_ancestor)
{
    class SetHasSelectedAncestor : public SceneAction
    {
    public:
        SetHasSelectedAncestor(EntityID id, bool has_selected_ancestor)
            : m_id(id), m_has_selected_ancestor(has_selected_ancestor) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(
                m_id)->set_has_selected_ancestor(m_has_selected_ancestor);
        }

    private:
        const EntityID m_id;
        const bool m_has_selected_ancestor;
    };

    dispatch(new SetHasSelectedAncestor(entity_id(), has_selected_ancestor));
}


void ObjectEntityProxy::set_parent(EntityID parent_id)
{
    class SetParent : public SceneAction
    {
    public:
        SetParent(EntityID id, EntityID parent_id)
            : m_id(id), m_parent_id(parent_id) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<ObjectEntity>(m_id)->set_parent(
                context.scene().get_entity<ObjectEntity>(m_parent_id));
        }

    private:
        const EntityID m_id, m_parent_id;
    };

    dispatch(new SetParent(entity_id(), parent_id));
}


void ObjectEntityProxy::set_parent(const ObjectEntityProxy &e)
{
    set_parent(e.entity_id());
}
