/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "OccupancyGrid2DProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../entities/OccupancyGrid2D.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


OccupancyGrid2DProxy::OccupancyGrid2DProxy(
    Renderer &renderer,
    float cell_size,
    const RGBColor &unoccupied_color,
    const RGBColor &occupied_color)
    : ObjectEntityProxy(renderer)
{
    class AddOccupancyGrid2D : public SceneAction
    {
    public:
        AddOccupancyGrid2D(
            EntityID id,
            float cell_size,
            const RGBColor &unoccupied_color,
            const RGBColor &occupied_color)
            : m_id(id),
              m_cell_size(cell_size),
              m_unoccupied_color(unoccupied_color),
              m_occupied_color(occupied_color) {}

        virtual void execute(SceneAction::Context &context)
        {
            Entity *e = new OccupancyGrid2D(
                m_cell_size,
                m_unoccupied_color,
                m_occupied_color,
                1024*256);

            try
            {
                context.scene().add_entity(m_id, e);
            }
            catch(...)
            {
                delete e;
                throw;
            }
        }

    private:
        const EntityID m_id;
        const float m_cell_size;
        const RGBColor m_unoccupied_color;
        const RGBColor m_occupied_color;
    };

    dispatch(
        new AddOccupancyGrid2D(
            entity_id(), cell_size, unoccupied_color, occupied_color));
}


void OccupancyGrid2DProxy::set_cells(
    const std::vector<std::pair<Vector2f, float> > &cells)
{
    class SetCells : public SceneAction
    {
    public:
        SetCells(
            EntityID id,
            const std::vector<std::pair<Vector2f, float> > &cells)
            : m_id(id), m_cells(cells) {}

        virtual void execute(SceneAction::Context &context)
        {
            for( size_t i = 0; i < m_cells.size(); ++i )
                context.scene().get_entity<OccupancyGrid2D>(m_id)->set_belief(
                    m_cells[i].first, m_cells[i].second);
        }

    private:
        const EntityID m_id;
        const std::vector<std::pair<Vector2f, float> > m_cells;
    };

    dispatch(new SetCells(entity_id(), cells));
}
