/*
 * Copyright Staffan Gimåker 2008-2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_RENDERER_API_RENDERER_HH_INCLUDED
#define PEEKABOT_RENDERER_API_RENDERER_HH_INCLUDED


#include "../Types.hh"
#include "../../Types.hh"


namespace peekabot
{
    namespace renderer
    {
        class SceneAction;
        class FrameCollector;
        class RenderEngine;

        namespace api
        {
            class EntityProxy;
            class TransformationGuidesProxy;

            class Renderer
            {
                friend class EntityProxy;

            public:
                Renderer();

                ~Renderer();

                void initialize();

                // Must be called with a valid OpenGL context
                void render(
                    unsigned int w, unsigned int h,
                    EntityID camera_id,
                    bool *visible_layers,
                    FrameCollector *collector = 0);

                // Must be called with a valid OpenGL context
                uint32_t pick(
                    unsigned int w, unsigned int h,
                    EntityID camera_id,
                    bool *visible_layers,
                    int x, int y, float sensitivity);

                bool has_pending_updates() const throw();

                // Must be called with a valid OpenGL context
                void process_updates();


                void set_transformation_guides_mode(TransformationGuidesMode mode);

                void set_transformation_guides_coord_sys(CoordinateSystem coord_sys);

                void set_transformation_guides_pivot(RotationCenterpoint pivot);

                void set_transformation_guides_axes(Axis axes);

            private:
                void dispatch(SceneAction *action);

            private:
                RenderEngine *m_engine;

                bool m_is_initialized;

                TransformationGuidesProxy *m_guides;
            };
        }
    }
}


#endif // PEEKABOT_RENDERER_API_RENDERER_HH_INCLUDED
