/*
 * Copyright Staffan Gimåker 2009.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "TransformationGuidesProxy.hh"
#include "../SceneAction.hh"
#include "../Scene.hh"
#include "../entities/TransformationGuides.hh"


using namespace peekabot;
using namespace peekabot::renderer;
using namespace peekabot::renderer::api;


TransformationGuidesProxy::TransformationGuidesProxy(Renderer &renderer)
    : EntityProxy(renderer)
{
    class AddGuides : public SceneAction
    {
    public:
        AddGuides(EntityID id) : m_id(id) {}

        virtual void execute(SceneAction::Context &context)
        {
            Entity *e = new TransformationGuides(context.scene_manager());
            try
            {
                context.scene().add_entity(m_id, e);
            }
            catch(...)
            {
                delete e;
                throw;
            }
        }

    private:
        const EntityID m_id;
    };

    dispatch(new AddGuides(entity_id()));
}


void TransformationGuidesProxy::set_mode(TransformationGuidesMode mode)
{
    class SetMode : public SceneAction
    {
    public:
        SetMode(EntityID id, TransformationGuidesMode mode)
            : m_id(id), m_mode(mode) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<TransformationGuides>(
                m_id)->set_mode(m_mode);
        }

    private:
        const EntityID m_id;
        const TransformationGuidesMode m_mode;
    };

    dispatch(new SetMode(entity_id(), mode));
}

void TransformationGuidesProxy::set_coord_sys(CoordinateSystem coord_sys)
{
    class SetCoordSys : public SceneAction
    {
    public:
        SetCoordSys(EntityID id, CoordinateSystem coord_sys)
            : m_id(id), m_coord_sys(coord_sys) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<TransformationGuides>(
                m_id)->set_coord_sys(m_coord_sys);
        }

    private:
        const EntityID m_id;
        const CoordinateSystem m_coord_sys;
    };

    dispatch(new SetCoordSys(entity_id(), coord_sys));
}


void TransformationGuidesProxy::set_pivot(RotationCenterpoint pivot)
{
    class SetPivot : public SceneAction
    {
    public:
        SetPivot(EntityID id, RotationCenterpoint pivot)
            : m_id(id), m_pivot(pivot) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<TransformationGuides>(
                m_id)->set_pivot(m_pivot);
        }

    private:
        const EntityID m_id;
        const RotationCenterpoint m_pivot;
    };

    dispatch(new SetPivot(entity_id(), pivot));
}


void TransformationGuidesProxy::set_axes(Axis axes)
{
    class SetAxes : public SceneAction
    {
    public:
        SetAxes(EntityID id, Axis axes)
            : m_id(id), m_axes(axes) {}

        virtual void execute(SceneAction::Context &context)
        {
            context.scene().get_entity<TransformationGuides>(
                m_id)->set_axes(m_axes);
        }

    private:
        const EntityID m_id;
        const Axis m_axes;
    };

    dispatch(new SetAxes(entity_id(), axes));
}
