/*
 * Copyright Staffan Gimåker 2007-2008.
 *
 * ---
 *
 * This file is part of peekabot.
 *
 * peekabot is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * peekabot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PEEKABOT_RENDERER_PRIMITIVES_MESH_ENTITIES_HH_INCLUDED
#define PEEKABOT_RENDERER_PRIMITIVES_MESH_ENTITIES_HH_INCLUDED


#include <boost/shared_ptr.hpp>

#include "../../Types.hh"
#include "../CullableEntity.hh"
#include "../VertexBased.hh"
#include "../LineBased.hh"
#include "../IndexBased.hh"



namespace peekabot
{
    namespace renderer
    {
        class Mesh;
        class Camera;

        class MeshEntity : public CullableEntity, 
                           public VertexBased
        {
        public:
            MeshEntity(boost::shared_ptr<Mesh> mesh) throw();

            virtual ~MeshEntity();

            virtual void get_renderables(PrepareRenderContext &context) const;

            virtual void set_vertices(
                const VertexBased::Vertices &vertices) throw();
                
            virtual void add_vertices(
                const VertexBased::Vertices &vertices) throw();

            virtual void get_vertices(VertexBased::Vertices &vertices) const throw();

            boost::shared_ptr<Mesh> get_mesh() throw();

            const boost::shared_ptr<Mesh> get_mesh() const throw();

        protected:
            void calculate_bvs(const VertexBased::Vertices &vertices) throw();
    
        protected:
            boost::shared_ptr<Mesh> m_mesh;
        };


        class PointMeshEntity : public MeshEntity
        {
        public:
            PointMeshEntity(boost::shared_ptr<Mesh> mesh) throw();

            PointMeshEntity(const PointMeshEntity &x) throw();

            virtual ~PointMeshEntity();

            virtual PointMeshEntity *clone() const throw();
        };


        class LineMeshEntity : public MeshEntity, 
                               public LineBased
        {
        public:
            LineMeshEntity(boost::shared_ptr<Mesh> mesh) throw();

            LineMeshEntity(const LineMeshEntity &x) throw();

            virtual ~LineMeshEntity();

            virtual LineMeshEntity *clone() const throw();

            virtual void set_line_style(
                float line_width, 
                uint16_t stipple_pattern, 
                int stipple_factor);
        };


        class TriMeshEntity : public MeshEntity,
                              public IndexBased
        {
        public:
            TriMeshEntity(boost::shared_ptr<Mesh> mesh) throw();

            TriMeshEntity(const TriMeshEntity &x) throw();

            virtual ~TriMeshEntity();

            virtual TriMeshEntity *clone() const throw();

            virtual void set_indices(const Indices &indices);

            virtual void add_indices(const Indices &indices);

            virtual void get_indices(Indices &indices) const;
        };
    }
}


#endif // PEEKABOT_RENDERER_PRIMITIVES_MESH_ENTITIES_HH_INCLUDED
